<?php

namespace Ems\Util;

class ArrayUtil {
    
    const COMMA_ARRAY_STRING_SEPARATOR  = ',';
    const ARRAY_ORDER_DESC              = 'DESC';
    const ARRAY_ORDER_ASC               = 'ASC';
    
    /**
     * Returns all items which are in the first array (alpha), but are not
     *in the second array (beta).
     *
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    public static function difference(
        array $alpha,
        array $beta
    ) : array {
        $difference = array_filter($alpha, function($item) use ($beta) {
            $inBeta = in_array($item, $beta);
            return !$inBeta;
        });

        return $difference;
    }

    /**
     * Returns the union between two arrays.
     *
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    public static function union(
        array $alpha,
        array $beta
    ) : array {
        $difference = self::difference($alpha, $beta);
        $union = array_merge($difference, $beta);

        return $union;
    }

    /**
     * Returns the intersection between two arrays.
     *
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    public static function intersection(
        array $alpha,
        array $beta
    ) : array {
        $intersection = [];

        foreach ($alpha as $alphaItem) {
            if (in_array($alphaItem, $beta)) {
                array_push($intersection, $alphaItem);
            }
        }

        return $intersection;
    }

    /**
     * Converts an array to string, adding commas around each item.
     *
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    public static function toStringSQL(
        array $array,
        $separator = ','
    ) : string {
        if (empty($array)) {
            return '';
        }

        $stringArray = $separator;

        foreach ($array as $item) {
            $stringArray .= $item . $separator;
        }

        return $stringArray;
    }

    public static function fromStringSQL(
        string  $string,
                $separator = self::COMMA_ARRAY_STRING_SEPARATOR,
        bool    $isSeparatorInBeginAndEnd = true
    ): array {
        // ",100,101,102," => ["", "100", "101", "102", ""]
        $items = explode($separator, $string);

        if (count($items) === 0) {
            return [];
        }
        
        if ($isSeparatorInBeginAndEnd) {
            $lastIndex = count($items) - 1;
    
            array_splice($items, 0, 1);
            array_splice($items, $lastIndex, 1);
        }


        return $items;
    }

    public function removeIndex(
        array $array,
        int $index
    ): array {
        unset($array[$index]);
        array_values($array);

        return $array;
    }


    public static function stringAplhabeticOrderByKey($array, $key, $order = self::ARRAY_ORDER_DESC) {
        $orderMultiplicator = $order == self::ARRAY_ORDER_ASC ? -1 : 1;
        usort($array, function($a, $b) use ($key, $orderMultiplicator) {
            return $orderMultiplicator * strcmp($a[$key], $b[$key]);
        });

        return $array;
    }

    public static function find(
        array    $array,
        callable $callback
    ) {
        $filteredItems = array_filter($array, $callback);
        $filteredItems = array_values($filteredItems);

        $item = $filteredItems[0] ?? false;

        return $item;
    }
}